package BAOBAB::ChromosomeWrapper;

# $Id: ChromosomeWrapper.pm,v 1.9 2010/03/05 16:29:26 rnl14480 Exp $

@ISA = qw(BAOBAB::Wrapper);

use strict;
use BAOBAB::Wrapper;
use BAOBAB::Chromosome;
use BAOBAB::Organism;
use BAOBAB::OrganismWrapper;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = {
		object => $a{object},
		dbh    => $a{dbh}
	};

	$c->SUPER::new(@_);
	return bless $r, $c;
}

sub write {
	my $s = shift;
	return unless $s->is_dirty;

	my $o   = $s->wrap;
	my $dbh = $s->connect;

	my ( $param, $whereclause ) = $s->whereclause;

	my $sel = $dbh->prepare(
		qq{
		SELECT * FROM chromosome $whereclause
	}
	) or die $dbh->errstr;

	$sel->execute(@$param) or die $sel->errstr;

	my $sth;
	if ( $sel->rows ) {
		$sth = $dbh->prepare(
			qq{
			UPDATE chromosome SET \
			oid = ?, 	\
			name = ?, 	\
			size = ?	\
			$whereclause
		}
		) or die $dbh->errstr;

		$sth->execute( $o->organism->id, $o->name, $o->size, @$param )
		  or die $sth->errstr;
	}
	else {

		$sth = $dbh->prepare(
			qq{
			INSERT INTO chromosome (oid, name, size) \
			VALUES (?,?,?)
		}
		) or die $sth->errstr;

		$sth->execute( $o->organism->id, $o->name, $o->size )
		  or die $sth->errstr;

		my $id = $dbh->{q{mysql_insertid}};
		$o->set_id($id);
		$BAOBAB::Chromosome::cache{$id} = $o unless $o->no_cache;
	}

	$dbh->commit or die $dbh->errstr;
	$sel->finish;
	$sth->finish;
	$s->clean;
}

sub list {
	my $s   = shift;
	my %a   = @_;
	my $dbh = $s->connect;

	my $organism = $a{organism};
	my $extra    = $a{extra};

	my $atom = qq{count(*)};
	$atom = qq{chromosome.id} if wantarray;

	my $sql = qq{SELECT $atom FROM chromosome};
	$sql .= qq{ WHERE oid = } . $organism->id if defined $organism;

	$sql .= qq{ $extra} if defined $extra;

	my $res = $dbh->selectcol_arrayref($sql) or die $dbh->errstr;

	return wantarray ? @$res : $res->[0];
}

sub iterator {
	my $s   = shift;
	my $dbh = $s->connect;

	my $sth = $dbh->prepare(
		qq{
		SELECT id FROM chromosome
	}
	) or die $dbh->errstr;
	$sth->execute();

	return sub {
		my ($id) = $sth->fetchrow_array();
		$sth->finish unless defined $id;
		return $id;
	  }
}

sub read {
	my $s = shift;
	my %a = @_;

	my $id       = $a{id};
	my $no_cache = $a{no_cache};

	my $o = undef;
	if ( defined $id && exists $BAOBAB::Chromosome::cache{$id} ) {
		$o = $BAOBAB::Chromosome::cache{$id};
	}
	else {
		my $dbh = $s->connect;
		my ( $param, $whereclause ) = $s->whereclause(%a);

		my $sql = qq{SELECT * FROM chromosome $whereclause};

		my $data = $dbh->selectrow_hashref( $sql, undef, @$param )
		  or die $dbh->errstr;

		if ( exists $BAOBAB::Chromosome::cache{ $data->{id} } ) {
			$o = $BAOBAB::Chromosome::cache{ $data->{id} };
		}
		else {

			my $ow = BAOBAB::OrganismWrapper->new( dbh => $dbh );

			$o = BAOBAB::Chromosome->new(
				id => $data->{id},
				organism =>
				  sub { $ow->read( id => $data->{oid}, no_cache => $no_cache ) }
				,
				name     => $data->{name},
				size     => $data->{size},
				no_cache => $no_cache
			);
			$o->clean;
		}

	}

	return $s->wrap($o);
}

sub whereclause {
	my $s = shift;
	my $o = $s->wrap;
	my %a = @_;
	my ( $param, $whereclause ) = ( [], '' );

	my $no_params = scalar(@_) ? 0 : 1;

	if ( defined $a{id} or ( $no_params and defined $o and defined $o->id ) ) {
		$whereclause = q{WHERE id = ?};
		$param = [ $a{id} || $o->id ];
	}
	elsif (
		( defined $a{name} and defined $a{oid} )
		or (    $no_params
			and defined $o
			and defined $o->name
			and defined $o->organism->id )
	  )
	{
		$whereclause = q{WHERE name = ? AND oid = ?};
		$param = [ $a{name} || $o->name, $a{oid} || $o->organism->id ];
	}
	else {
		$whereclause = q{WHERE 1 = 0};
	}

	return ( $param, $whereclause );
}

sub DESTROY {
	(shift)->SUPER::DESTROY(@_);
}

1;
