package BAOBAB::EvaluationWrapper;

# $Id: EvaluationWrapper.pm,v 1.4 2010/02/24 00:41:09 rnl14480 Exp $

@ISA = qw(BAOBAB::Wrapper);

use strict;
use BAOBAB::Wrapper;
use BAOBAB::Evaluation;
use BAOBAB::PrecursorWrapper;
use BAOBAB::MeasureWrapper;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = {
		object => $a{object},
		dbh    => $a{dbh}
	};

	$c->SUPER::new(@_);
	return bless $r, $c;
}

sub write {
	my $s = shift;
	return unless $s->is_dirty;

	my $o   = $s->wrap;
	my $dbh = $s->connect;

	my ( $param, $whereclause ) = $s->whereclause;

	my $sel = $dbh->prepare(
		qq{
		SELECT * FROM evaluation $whereclause
	}
	) or die $dbh->errstr;

	$sel->execute(@$param) or die $sel->errstr;

	my $sth;
	if ( $sel->rows ) {
		$sth = $dbh->prepare(
			qq{
			UPDATE evaluation SET \
			pid = ?, 	\
			mid = ?, 	\
			value = ?	\
			$whereclause
		}
		) or die $dbh->errstr;

		$sth->execute( $o->precursor->id, $o->measure->id, $o->value, @$param )
		  or die $sth->errstr;
	}
	else {

		$sth = $dbh->prepare(
			qq{
			INSERT INTO evaluation (pid, mid, value) \
			VALUES (?,?,?)
		}
		) or die $sth->errstr;

		$sth->execute( $o->precursor->id, $o->measure->id, $o->value )
		  or die $sth->errstr;
	}

	$dbh->commit or die $dbh->errstr;
	$sel->finish;
	$sth->finish;
	$s->clean;

}

sub list {
	my $s   = shift;
	my $dbh = $s->connect;

	my $res = $dbh->selectcol_hashref(
		qq{
		SELECT pid, mid FROM evaluation
		}
	) or die $dbh->errstr;

	return wantarray ? @$res : $res;
}

sub iterator {
	my $s     = shift;
	my $dbh   = $s->connect;
	my %a     = @_;
	my $start = $a{start};
	my $limit = $a{limit};
	my $extra = $a{extra};

	my $extraclause = qq{WHERE $extra} if defined $extra;
	if ( defined $start && defined $limit ) {
		$extraclause .= qq{ LIMIT $limit OFFSET $start};
	}

	$extraclause = '' unless defined $extraclause;
	my $select_clause = qq { SELECT pid, mid FROM evaluation $extraclause };

	my $sth = $dbh->prepare($select_clause) or die $dbh->errstr;
	$sth->execute();

	return sub {
		my ( $pid, $mid ) = $sth->fetchrow_array();
		$sth->finish unless defined $pid && defined $mid;
		return wantarray ? ( $pid, $mid ) : [ $pid, $mid ];
	  }
}

sub read {
	my $s = shift;
	my %a = @_;

	my $pid      = $a{pid};
	my $mid      = $a{mid};
	my $id       = qq{$pid:$mid};
	my $no_cache = $a{no_cache};

	my $o = undef;
	if ( defined $id && exists $BAOBAB::Evaluation::cache{$id} ) {
		$o = $BAOBAB::Evaluation::cache{$id};
	}
	else {

		my $dbh = $s->connect;
		my ( $param, $whereclause ) = $s->whereclause(%a);

		my $data = $dbh->selectrow_hashref(
			qq{
				SELECT * FROM evaluation $whereclause
			}, undef, @$param
		) or die $dbh->errstr;

		my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );
		my $mw = BAOBAB::MeasureWrapper->new( dbh => $dbh );

		$o = BAOBAB::Evaluation->new(
			precursor => sub {
				$pw->read(
					id       => $data->{pid},
					no_cache => $no_cache
				);
			},
			measure  => sub { $mw->read( id => $data->{mid} ) },
			value    => $data->{value},
			no_cache => $no_cache
		);
		$o->clean;
	}

	return $s->wrap($o);
}

sub whereclause {
	my $s = shift;
	my $o = $s->wrap;
	my %a = @_;
	my ( $param, $whereclause ) = ( [], '' );

	my $no_params = scalar(@_) ? 0 : 1;

	my $pidclause =
	  (
		defined( $a{pid} )
		  or $no_params and defined($o) and defined( $o->precursor->id )
	  )
	  ? 1
	  : 0;
	my $midclause =
	  (
		defined( $a{mid} )
		  or $no_params and defined($o) and defined( $o->measure->id )
	  )
	  ? 1
	  : 0;

	if ( $pidclause && $midclause ) {
		$whereclause = q{WHERE pid = ? AND mid = ?};
		$param = [ $a{pid} || $o->precursor->id, $a{mid} || $o->measure->id ];
	}
	else {
		$whereclause = q{WHERE 1 = 0};
	}

	return ( $param, $whereclause );

}

sub DESTROY {
	(shift)->SUPER::DESTROY(@_);
}

1;
