package BAOBAB::HMM;

#$Id: HMM.pm,v 1.8 2010/06/17 16:49:20 rnl14480 Exp $

use strict;
use BAOBAB::Random qw(random);
our @ALPHA = qw(A C T G);
our %ALPHA = ();
$ALPHA{$_} = 1 for @ALPHA;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = { _rg => random(100), };
	return bless $r, $c;
}

sub train {
	my $s         = shift;
	my %a         = @_;
	my $sequences = $a{seqs} || $a{sequences};
	my %i         = ();
	my %t         = ();

	my @slen;
	foreach my $seq (@$sequences) {
		next unless $seq;
		$seq =~ y/uU/tT/;
		my ( $cur, $prev, $pos );
		$pos = 0;
		$prev = uc( substr $seq, $pos, 1 );
		$i{$prev}++;
		for ( $pos = 1 ; $pos < length($seq) ; $pos++ ) {
			$cur = uc( substr $seq, $pos, 1 );
			$i{$cur}++ if $ALPHA{$cur};
			$t{$prev}{$cur}++ if $ALPHA{$cur} and $ALPHA{$prev};
			$prev = $cur;
		}
		push @slen, length($seq);
	}

	my $slen = 0;
	$slen += $_ for @slen;
	$slen /= scalar(@$sequences);

	my $pcount = $slen * scalar(@ALPHA);
	$pcount = 1 / $pcount if $pcount;
	$pcount = 1 unless $pcount;

	my $ti = 0;
	foreach my $n (@ALPHA) {
		$i{$n} += $pcount;    # init pseudocounts
		$ti += $i{$n};
	}
	foreach my $n (@ALPHA) {
		$i{$n} /= $ti;
	}

	$pcount *= 1 / scalar(@ALPHA);
	my $tt;
	foreach my $n1 (@ALPHA) {
		foreach my $n2 (@ALPHA) {
			$t{$n1}{$n2} += $pcount;        # transition pseudocounts
			$tt->{$n1}   += $t{$n1}{$n2};
		}
	}
	foreach my $n1 (@ALPHA) {
		foreach my $n2 (@ALPHA) {
			$t{$n1}{$n2} /= $tt->{$n1};
		}
	}

	$s->{model}{init}       = \%i;
	$s->{model}{transition} = \%t;
}

sub generate {
	my $s      = shift;
	my %a      = @_;
	my $model  = $s->{model};
	my $len    = $a{len};
	my $seq    = '';
	my $is_rna = $a{rna} || $a{RNA};

	return $seq if $len == 0;

	my $i  = $model->{init};
	my $pr = $s->prob;
	my @k  = keys %$i;
	my ( $p, $c ) = ( undef, undef );
	my $ti = 0;

	#print STDERR "Selection init pr = $pr\n";

	foreach my $k (@k) {

		#print STDERR "p{$k} = " . ( $ti + $i->{$k} * 100 ) . " > $pr\n";
		if ( $pr < $ti + $i->{$k} * 100 ) {
			$c = $k;
			last;
		}
		else {
			$ti += $i->{$k} * 100;
		}
	}
	$seq .= $c;
	$len--;

	#print STDERR "seq = $seq, remains = $len\n";

	$p = $c;
	my $t = $model->{transition};
	while ($len) {
		my @k  = keys %{ $t->{$p} };
		my $tt = 0;
		my $pr = $s->prob;
		foreach my $k (@k) {

			#	print STDERR "p{$p}{$k} = "
			#	  . ( $tt + $t->{$p}{$k} * 100 )
			#	  . " > $pr\n";
			if ( $pr < $tt + $t->{$p}{$k} * 100 ) {
				$c = $k;
				last;
			}
			else {
				$tt += $t->{$p}{$k} * 100;
			}
		}
		$seq .= $c;
		$len--;
		$p = $c;
	}

	$seq =~ y/Tt/Uu/ if $is_rna;
	return $seq;
}

sub prob {
	my $s = shift;
	return $s->{_rg}->();
}

1;

