package BAOBAB::Index;

use Fcntl qw(:seek);
use strict;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = {
		_file     => $a{file},
		_hasindex => undef,
		_index    => undef,
	};

	bless $r, $c;
	$r->_init;
	return $r;
}

sub _init {
	my $s = shift;
	unless ( $s->_hasindex ) {
		$s->compute_index;
		$s->write_index;
	}
	else {
		$s->load_index;
	}
	return;
}

sub _findexname {
	my $s = shift;
	return $s->{_findexname} = $s->{_file} . ".idx";
}

sub _hasindex {
	my $s = shift;
	return $s->{_hasindex} = -e $s->_findexname;
}

sub index {
	my $s   = shift;
	my $idx = $s->{_index};
	return wantarray ? @$idx : $idx;
}

sub file {
	my $s = shift;
	return $s->{_file};
}

sub write_index {
	my $s = shift;
	my $idx;
	my $finame = $s->_findexname;

	open( $idx, ">$finame" ) or die "Cannot open $finame : $!\n";
	print $idx $_->[0] . ":" . $_->[1] . "\n" for $s->index;
	close($idx);
}

sub load_index {
	my $s     = shift;
	my @index = ();
	my $idx;
	my $finame = $s->_findexname;

	open( $idx, $finame ) or die "Cannot open $finame : $!\n";
	while (<$idx>) {
		chomp;
		my ( $start, $stop ) = split /:/;
		push @index, [ $start, $stop ];
	}

	$s->{_index} = \@index;
	return wantarray ? @index : \@index;
}

sub compute_index {
	my $s = shift;
	my $file;
	my $fname = $s->{_file};

	open( $file, $fname ) or die "Cannot open $fname : $!\n";

	my $state = 0;
	my @sequences;
	my ( $start, $stop );

	while ( !eof($file) ) {
		my $c;
		read( $file, $c, 1 );

		if ( $c eq '>' ) {
			if ( $state == 3 ) {    # no \n between seqs
				push @sequences, [ $start, $stop ];
				$start = $stop = undef;
			}
			$state = 1;
		}
		elsif ( $state == 1 ) {
			if ( $c eq "\n" ) {
				$state = 2;
				$start = tell($file);
			}
		}
		elsif ( $state == 2 ) {
			$state = 3 if $c eq "\n";
			$stop =
			  tell($file); #just in cases eof follows with or without a new line
		}
		elsif ( $state == 3 && $c eq "\n" ) {
			$state = 0;
			$stop  = tell($file);
			push @sequences, [ $start, $stop ];
			$start = $stop = undef;
		}
		elsif ( $state == 3 && $c ne "\n" ) {
			$state = 2;
		}
	}

	# in case there is no empty line at the end
	if ( defined $start && defined $stop ) {
		push @sequences, [ $start, $stop ];
	}

	close($file);

	$s->{_index} = \@sequences;
	return wantarray ? @sequences : \@sequences;

}

sub get_region {
	my $s      = shift;
	my %a      = @_;
	my $seq    = $a{sequence};
	my $offset = $a{offset};
	my $len    = $a{len};
	my $strand = $a{strand};

  #print STDERR "get_region(SEQ=$seq,OFFSET=$offset,LEN=$len,STRAND=$strand)\n";

	my $idx   = $s->index;
	my $fname = $s->file;
	my $file;
	open( $file, $fname ) or die "Cannot open $fname : $!\n";

	my $b = $idx->[$seq];
	my ( $start, $stop ) = @$b;
	if ( $offset > $stop - $start ) {
		die "Offset ($offset) too big\n";
	}

	seek( $file, $start,  SEEK_SET );
	seek( $file, $offset, SEEK_CUR );

	my $region;
	read( $file, $region, $len );
	close($file);

	if ( $strand eq 'R' ) {
		$region = $s->_reverse_complement($region);
	}
	$region =~ y/Tt/Uu/;

	return $region;
}

sub _reverse_complement {
	my $s   = shift;
	my $seq = shift;
	my @seq = split //, $seq;
	@seq = reverse(@seq);
	$seq = join( '', @seq );
	$seq =~ y/ATGCatgc/TACGtacg/;
	return $seq;
}

1;
