package BAOBAB::MatureWrapper;

# $Id: MatureWrapper.pm,v 1.5 2010/02/24 00:41:09 rnl14480 Exp $

@ISA = qw(BAOBAB::Wrapper);

use strict;
use BAOBAB::Wrapper;
use BAOBAB::Mature;
use BAOBAB::Precursor;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = {
		object => $a{object},
		dbh    => $a{dbh},
	};

	$c->SUPER::new(@_);
	return bless $r, $c;
}

sub write {
	my $s = shift;
	return unless $s->is_dirty;

	my $o   = $s->wrap;
	my $dbh = $s->connect;

	my ( $param, $whereclause ) = $s->whereclause;

	my $sel = $dbh->prepare(
		qq{
		SELECT * FROM mature $whereclause
	}
	) or die $dbh->errstr;

	$sel->execute(@$param) or die $sel->errstr;

	my $sth;
	if ( $sel->rows ) {
		$sth = $dbh->prepare(
			qq{
			UPDATE mature SET \
			pid = ?, 	\
			position = ?, 	\
			length = ?,	\
			designation = ? \
			$whereclause
		}
		) or die $dbh->errstr;

		$sth->execute( $o->precursor->id, $o->position, $o->length,
			$o->designation, @$param )
		  or die $sth->errstr;
	}
	else {

		$sth = $dbh->prepare(
			qq{
			INSERT INTO mature (pid, position, length, designation) \
			VALUES (?,?,?,?)
		}
		) or die $sth->errstr;

		$sth->execute( $o->precursor->id, $o->position, $o->length,
			$o->designation )
		  or die $sth->errstr;

		my $id = $dbh->{q{mysql_insertid}};
		$o->set_id($id);
		$BAOBAB::Mature::cache{$id} = $o unless $o->no_cache;
	}

	$dbh->commit or die $dbh->errstr;
	$sel->finish;
	$sth->finish;
	$s->clean;

}

sub list {
	my $s   = shift;
	my %a   = @_;
	my $dbh = $s->connect;

	my $whereclause = '';
	my $precursor   = $a{precursor};

	if ( defined $precursor ) {
		my $id = $precursor->id;
		$whereclause = qq{WHERE pid = $id};
	}

	my $res = $dbh->selectcol_arrayref(
		qq{
		SELECT id FROM mature
		$whereclause
		}
	) or die $dbh->errstr;

	return wantarray ? @$res : $res;
}

sub iterator {
	my $s   = shift;
	my $dbh = $s->connect;

	my $sth = $dbh->prepare(
		qq{
		SELECT id FROM mature
		}
	) or die $dbh->errstr;
	$sth->execute();

	return sub {
		my ($id) = $sth->fetchrow_array();
		$sth->finish unless defined $id;
		return $id;
	  }
}

sub read {
	my $s = shift;
	my %a = @_;

	my $id       = $a{id};
	my $no_cache = $a{no_cache};

	my $o = undef;
	if ( defined $id && exists $BAOBAB::Mature::cache{$id} ) {
		$o = $BAOBAB::Mature::cache{$id};
	}
	else {

		my $dbh = $s->connect;
		my ( $param, $whereclause ) = $s->whereclause(%a);

		my $data = $dbh->selectrow_hashref(
			qq{
				SELECT * FROM mature $whereclause
			}, undef, @$param
		) or die $dbh->errstr;

		my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );

		$o = BAOBAB::Mature->new(
			id          => $data->{id},
			precursor   => sub { $pw->read( id => $data->{pid} ) },
			position    => $data->{position},
			length      => $data->{length},
			designation => $data->{designation},
			no_cache    => $no_cache
		);
		$o->clean;
	}
	return $s->wrap($o);
}

sub whereclause {
	my $s = shift;
	my $o = $s->wrap;
	my %a = @_;
	my ( $param, $whereclause ) = ( [], '' );

	my $no_params = scalar(@_) ? 0 : 1;

	if ( defined $a{id} or $no_params and defined $o and defined $o->id ) {
		$whereclause = q{WHERE id = ?};
		$param = [ $a{id} || $o->id ];
	}
	elsif (
		(
			    defined $a{precursor}
			and defined $a{position}
			and defined $a{length}
		)
		or (    $no_params
			and defined $o
			and defined $o->precursor
			and defined $o->precursor->id
			and defined $o->position
			and defined $o->length )
	  )
	{
		$whereclause = q{WHERE pid = ? AND position = ? AND length = ?};

		my $pid = undef;
		$pid = $a{precursor}->id if defined $a{precursor};
		my $position = $a{position};
		$position = $o->position unless defined $position;

		$param =
		  [ $pid || $o->precursor->id, $position, $a{length} || $o->length ];

	}
	else {
		$whereclause = q{WHERE 1 = 0 };
	}

	return ( $param, $whereclause );

}

sub DESTROY {
	(shift)->SUPER::DESTROY(@_);
}

1;
