package BAOBAB::MeasureWrapper;

# $Id: MeasureWrapper.pm,v 1.4 2010/02/24 00:41:09 rnl14480 Exp $

@ISA = qw(BAOBAB::Wrapper);

use strict;
use BAOBAB::Wrapper;
use BAOBAB::Measure;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = {
		object => $a{object},
		dbh    => $a{dbh}
	};

	$c->SUPER::new(@_);
	return bless $r, $c;
}

sub write {
	my $s = shift;
	return unless $s->is_dirty;

	my $o   = $s->wrap;
	my $dbh = $s->connect;

	my ( $param, $whereclause ) = $s->whereclause;

	my $sel = $dbh->prepare(
		qq{
		SELECT * FROM measure $whereclause
	}
	) or die $dbh->errstr;

	$sel->execute(@$param) or die $sel->errstr;

	my $sth;
	if ( $sel->rows ) {
		$sth = $dbh->prepare(
			qq{
			UPDATE measure SET \
			name = ?, 	\
			description = ? \
			$whereclause
		}
		) or die $dbh->errstr;

		$sth->execute( $o->name, $o->description, @$param )
		  or die $sth->errstr;
	}
	else {

		$sth = $dbh->prepare(
			qq{
			INSERT INTO measure (name, description) \
			VALUES (?,?)
		}
		) or die $sth->errstr;

		$sth->execute( $o->name, $o->description )
		  or die $sth->errstr;

		my $id = $dbh->{q(mysql_insertid)};
		$o->set_id($id);
		$BAOBAB::Measure::cache{$id} = $o unless $o->no_cache;
	}

	$dbh->commit or die $dbh->errstr;
	$sel->finish;
	$sth->finish;
	$s->clean;
}

sub list {
	my $s   = shift;
	my $dbh = $s->connect;

	my $res = $dbh->selectcol_arrayref(
		qq{
		SELECT id FROM measure
		}
	) or die $dbh->errstr;

	return wantarray ? @$res : $res;
}

sub iterator {
	my $s   = shift;
	my $dbh = $s->connect;

	my $sth = $dbh->prepare(
		qq{
		SELECT id FROM measure
		}
	) or die $dbh->errstr;
	$sth->execute();

	return sub {
		my ($id) = $sth->fetchrow_array();
		$sth->finish unless defined $id;
		return $id;
	  }
}

sub read {
	my $s = shift;
	my %a = @_;

	my $id       = $a{id};
	my $no_cache = $a{no_cache};

	my $o = undef;
	if ( defined $id && exists $BAOBAB::Measure::cache{$id} ) {
		$o = $BAOBAB::Measure::cache{$id};
	}
	else {

		my $dbh = $s->connect;
		my ( $param, $whereclause ) = $s->whereclause(%a);

		my $data = $dbh->selectrow_hashref(
			qq{
				SELECT * FROM measure $whereclause
			}, undef, @$param
		) or die $dbh->errstr;

		if ( exists $BAOBAB::Measure::cache{ $data->{id} } ) {
			$o = $BAOBAB::Measure::cache{ $data->{id} };
		}
		else {

			$o = BAOBAB::Measure->new(
				id          => $data->{id},
				name        => $data->{name},
				description => $data->{description},
				no_cache    => $no_cache
			);
			$o->clean;
		}
	}
	return $s->wrap($o);
}

sub whereclause {
	my $s = shift;
	my $o = $s->wrap;
	my %a = @_;
	my ( $params, $whereclause ) = ( [], '' );

	my $no_params = scalar(@_) ? 0 : 1;

	if ( defined $a{id} or $no_params and defined $o and defined $o->id ) {
		$whereclause = q{WHERE id = ?};
		$params = [ $a{id} || $o->id ];
	}
	elsif ( defined $a{name}
		or ( $no_params and defined $o and defined $o->name ) )
	{
		$whereclause = q{WHERE name = ?};
		$params = [ $a{name} || $o->name ];
	}
	else {
		$whereclause = q{WHERE 1 = 0};
	}

	return ( $params, $whereclause );

}

sub DESTROY {
	(shift)->SUPER::DESTROY(@_);
}

1;
