package BAOBAB::OrganismWrapper;

# $Id: OrganismWrapper.pm,v 1.6 2010/06/06 18:11:46 rnl14480 Exp $

@ISA = qw(BAOBAB::Wrapper);

use strict;
use BAOBAB::Wrapper;
use BAOBAB::Organism;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = {
		object => $a{object},
		dbh    => $a{dbh}
	};

	$c->SUPER::new(@_);
	return bless $r, $c;
}

sub write {
	my $s = shift;
	return unless $s->is_dirty;

	my $o   = $s->wrap;
	my $dbh = $s->connect;

	my ( $param, $whereclause ) = $s->whereclause;

	my $sel = $dbh->prepare(
		qq{
		SELECT * FROM organism $whereclause
	}
	) or die $dbh->errstr;

	$sel->execute($param) or die $sel->errstr;

	my $sth;
	if ( $sel->rows ) {
		$sth = $dbh->prepare(
			qq{
			UPDATE organism SET \
			genus = ?, 	\
			species = ?, 	\
			revision = ?,	\
			shortname = ?	\
			$whereclause
		}
		) or die $dbh->errstr;

		$sth->execute( $o->genus, $o->species, $o->revision, $o->shortname,
			$param )
		  or die $sth->errstr;

	}
	else {

		$sth = $dbh->prepare(
			qq{
			INSERT INTO organism (genus, species, revision, shortname) \
			VALUES (?,?,?,?)
		}
		) or die $sth->errstr;

		$sth->execute( $o->genus, $o->species, $o->revision, $o->shortname )
		  or die $sth->errstr;

		my $id = $dbh->{q{mysql_insertid}};
		$o->set_id($id);
		$BAOBAB::Organism::cache{$id} = $o unless $o->{no_cache};
	}

	$dbh->commit or die $dbh->errstr;
	$sel->finish;
	$sth->finish;
	$s->clean;
}

sub list {
	my $s   = shift;
	my $dbh = $s->connect;

	my $res = $dbh->selectcol_arrayref(
		qq{
		SELECT id FROM organism
	}
	) or die $dbh->errstr;

	return wantarray ? @$res : $res;
}

sub iterator {
	my $s     = shift;
	my $dbh   = $s->connect;
	my %a     = @_;
	my $extra = $a{extra};

	my $extraclause = qq{ $extra} if defined $extra;

	my $sth = $dbh->prepare(
		qq{
		SELECT id FROM organism $extraclause
	}
	) or die $dbh->errstr;
	$sth->execute();

	return sub {
		my ($id) = $sth->fetchrow_array();
		$sth->finish unless defined $id;
		return $id;
	  }
}

sub read {
	my $s = shift;
	my %a = @_;

	my $id       = $a{id};
	my $no_cache = $a{no_cache};

	my $o = $s->wrap;
	if ( defined $id && exists $BAOBAB::Organism::cache{$id} ) {
		$o = $BAOBAB::Organism::cache{$id};
	}
	else {

		my $dbh = $s->connect;
		my ( $param, $whereclause ) = $s->whereclause(%a);

		#print STDERR "input = ".join(":",@_)."\n";
		#print STDERR "param=$param, whereclause=$whereclause\n";

		my $data = $dbh->selectrow_hashref(
			qq{
			SELECT * FROM organism $whereclause
		}, undef, $param
		) or die $dbh->errstr;

		if ( exists $BAOBAB::Organism::cache{ $data->{id} } ) {
			$o = $BAOBAB::Organism::cache{ $data->{id} };
		}
		else {

			$o = BAOBAB::Organism->new(
				id        => $data->{id},
				genus     => $data->{genus},
				species   => $data->{species},
				revision  => $data->{revision},
				shortname => $data->{shortname},
				no_cache  => $no_cache
			);
			$o->clean;
		}
	}

	return $s->wrap($o);
}

sub whereclause {
	my $s = shift;
	my %a = @_;
	my $o = $s->wrap;
	my ( $param, $whereclause ) = ( undef, '' );

	my $no_params = scalar(@_) ? 0 : 1;

	if ( defined $a{id} or $no_params and defined $o and defined $o->id ) {
		$whereclause = q{WHERE id = ?};
		$param = $a{id} || $o->id;
	}
	elsif ( defined $a{shortname}
		or $no_params and defined $o and defined $o->shortname )
	{
		$whereclause = q{WHERE shortname = ?};
		$param = $a{shortname} || $o->shortname;
	}
	else {
		$whereclause = q{WHERE 1= 0};
	}

	return ( $param, $whereclause );
}

sub DESTROY {
	(shift)->SUPER::DESTROY(@_);
}

1;
