package BAOBAB::OverlapWrapper;

# $Id: OverlapWrapper.pm,v 1.6 2010/06/17 16:47:03 rnl14480 Exp $

@ISA = qw(BAOBAB::Wrapper);

use strict;
use BAOBAB::Wrapper;
use BAOBAB::Overlap;
use BAOBAB::Precursor;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = {
		object => $a{object},
		dbh    => $a{dbh}
	};

	$c->SUPER::new(@_);
	return bless $r, $c;
}

sub write {
	my $s = shift;
	return unless $s->is_dirty;

	my $o   = $s->wrap;
	my $dbh = $s->connect;

	my ( $param, $whereclause ) = $s->whereclause;

	my $sel = $dbh->prepare(
		qq{
		SELECT * FROM overlap $whereclause
	}
	) or die $dbh->errstr;

	$sel->execute(@$param) or die $sel->errstr;

	my $sth;
	if ( $sel->rows ) {
		$sth = $dbh->prepare(
			qq{
			UPDATE overlap SET \
			kpid = ?, 	\
			cpid = ?, 	\
			type = ?,	\
			pc = ?		\
			$whereclause
		}
		) or die $dbh->errstr;

		$sth->execute( $o->known->id, $o->candidate->id, $o->type, $o->pc,
			@$param )
		  or die $sth->errstr;
	}
	else {

		$sth = $dbh->prepare(
			qq{
			INSERT INTO overlap (kpid, cpid, type, pc) \
			VALUES (?,?,?,?)
		}
		) or die $sth->errstr;

		$sth->execute( $o->known->id, $o->candidate->id, $o->type, $o->pc )
		  or die $sth->errstr;
	}

	$dbh->commit or die $dbh->errstr;
	$sel->finish;
	$sth->finish;
	$s->clean;

}

sub list {
	my $s     = shift;
	my $dbh   = $s->connect;
	my %a     = @_;
	my $known = $a{known};

	my $extraclause;
	if ( defined $known ) {
		my $kpid = $known->id;
		$extraclause = qq{WHERE kpid = $kpid};
	}
	my $res = $dbh->selectall_arrayref(
		qq{
		SELECT kpid, cpid FROM overlap $extraclause
		}
		, { Slice => {} }
	) or die $dbh->errstr;

	return wantarray ? @$res : $res;
}

sub iterator {
	my $s     = shift;
	my $dbh   = $s->connect;
	my %a     = @_;
	my $known = $a{known};

	my $extraclause;
	if ( defined $known ) {
		my $kpid = $known->id;
		$extraclause = qq{WHERE kpid = $kpid};
	}

	my $sth = $dbh->prepare(
		qq{
		SELECT kpid, cpid FROM overlap $extraclause
		}
	) or die $dbh->errstr;
	$sth->execute();

	return sub {
		my ( $kpid, $cpid ) = $sth->fetchrow_array();
		$sth->finish unless defined $kpid && defined $cpid;
		return wantarray ? ( $kpid, $cpid ) : [ $kpid, $cpid ];
	  }
}

sub read {
	my $s = shift;
	my %a = @_;

	my $kpid     = $a{kpid};
	my $cpid     = $a{cpid};
	my $id       = qq{$kpid:$cpid};
	my $no_cache = $a{no_cache};

	my $o = undef;
	if ( defined $id && exists $BAOBAB::Overlap::cache{$id} ) {
		$o = $BAOBAB::Overlap::cache{$id};
	}
	else {

		my $dbh = $s->connect;
		my ( $param, $whereclause ) = $s->whereclause(%a);

		my $data = $dbh->selectrow_hashref(
			qq{
				SELECT * FROM overlap $whereclause
			}, undef, @$param
		) or die $dbh->errstr;

		my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );

		$o = BAOBAB::Overlap->new(
			known =>
			  sub { $pw->read( id => $data->{kpid}, no_cache => $no_cache ) },
			candidate =>
			  sub { $pw->read( id => $data->{cpid}, no_cache => $no_cache ) },
			type     => $data->{type},
			pc       => $data->{pc},
			no_cache => $no_cache
		);
		$o->clean;
	}

	return $s->wrap($o);
}

sub whereclause {
	my $s = shift;
	my $o = $s->wrap;
	my %a = @_;
	my ( $param, $whereclause ) = ( [], '' );

	my $no_params = scalar(@_) ? 0 : 1;

	my $kpidclause =
	  (
		defined( $a{kpid} )
		  or $no_params and defined($o) and defined( $o->known->id )
	  )
	  ? 1
	  : 0;
	my $cpidclause =
	  (
		defined( $a{cpid} )
		  or $no_params and defined($o) and defined( $o->candidate->id )
	  )
	  ? 1
	  : 0;

	if ( $kpidclause && $cpidclause ) {
		$whereclause = q{WHERE kpid = ? AND cpid = ?};
		$param = [ $a{kpid} || $o->known->id, $a{cpid} || $o->candidate->id ];
	}
	else {
		$whereclause = q{WHERE 1 = 0};
	}

	return ( $param, $whereclause );

}

sub DESTROY {
	(shift)->SUPER::DESTROY(@_);
}

1;
