package BAOBAB::Precursor;

# $Id: Precursor.pm,v 1.12 2010/06/17 16:47:03 rnl14480 Exp $

@ISA = qw(BAOBAB::DataObject);

use strict;
use BAOBAB::DataObject;
use RNA;

our %cache = ();

sub new {
	my $c = shift;
	my %a = @_;

	my $r = {
		id          => $a{id},
		chromosome  => $a{chromosome},
		sequence    => $a{sequence},
		structure   => $a{structure},
		mfe         => $a{mfe},
		start       => $a{start},
		stop        => $a{stop},
		strand      => $a{strand},
		category    => $a{category},
		score       => $a{score},
		marked      => $a{marked},
		designation => $a{designation},
		no_cache    => $a{no_cache}
	};

	bless $r, $c;
	$r = $r->SUPER::new();
	$cache{ $r->{id} } = $r if defined $a{id} && !$a{no_cache};
	return $r;
}

sub chromosome {
	my $s = shift;
	my $o = $s->{chromosome};
	$o = $o->() if ref($o) eq 'CODE';
	return $o;
}

sub organism {
	my $s = shift;
	return $s->chromosome->organism;
}

sub sequence {
	my $s = shift;
	my $v = shift;
	if ( defined $v ) {
		$s->{sequence}  = $v;
		$s->{structure} = undef;
		$s->dirty;
	}
	return $s->{sequence};
}

sub structure {
	my $s = shift;
	my $v = shift;
	if ( defined $v ) {
		$s->{structure} = $v;
		$s->dirty;
	}
	unless ( defined $s->{structure} ) {
		if ( defined $s->{sequence} ) {
			my ( $st, $e ) = RNA::fold( $s->{sequence} );
			$s->{structure} = $st;
			$s->{mfe}       = $e;
			$s->dirty;
		}
	}
	return $s->{structure};
}

sub mfe {
	my $s = shift;
	my $v = shift;
	if ( defined $v ) {
		$s->{mfe} = $v;
		$s->dirty;
	}
	return $s->{mfe};
}

sub start {
	my $s = shift;
	my $v = shift;
	if ( defined $v ) {
		$s->{start} = $v;
		$s->dirty;
	}
	return $s->{start};
}

sub stop {
	my $s = shift;
	my $v = shift;
	if ( defined $v ) {
		$s->{stop} = $v;
		$s->dirty;
	}
	return $s->{stop};
}

sub strand { return (shift)->{strand}; }

sub category {
	my $s = shift;
	my $v = shift;
	if ( defined $v ) {
		$s->{category} = $v;
		$s->dirty;
	}
	return $s->{category};
}

sub score {
	my $s = shift;
	my $v = shift;
	if ( defined $v ) {
		$s->{score} = $v;
		$s->dirty;
	}
	return $s->{score};
}

sub marked {
	my $s = shift;
	my $v = shift;

	#print STDERR "Setting to $v\n" if defined $v;
	if ( defined $v ) {
		if ( $s->{marked} != $v ) {

			#	print STDERR "CHANGED cause ".$s->{marked}." != $v\n";
			$s->{marked} = $v;
			$s->dirty;
		}
		else {

			#	print STDERR "No change cause ".$s->{marked}." == $v\n";
		}
	}
	return $s->{marked};
}

sub designation {
	my $s = shift;
	my $v = shift;
	if ( defined $v ) {
		$s->{designation} = $v;
		$s->dirty;
	}
	return $s->{designation};
}

sub length {
	my $s = shift;
	return length( $s->sequence );
}

1;
