package BAOBAB::PrecursorWrapper;

# $Id: PrecursorWrapper.pm,v 1.20 2010/06/17 16:47:03 rnl14480 Exp $

@ISA = qw(BAOBAB::Wrapper);

use strict;
use BAOBAB::Wrapper;
use BAOBAB::Precursor;
use BAOBAB::Chromosome;
use BAOBAB::ChromosomeWrapper;
use BAOBAB::Organism;

sub new {
	my $c = shift;
	my %a = @_;
	my $r = {
		object    => $a{object},
		dbh       => $a{dbh},
		no_update => $a{no_update}
	};

	$c->SUPER::new(@_);
	return bless $r, $c;
}

sub write {
	my $s = shift;
	return unless $s->is_dirty;

	my $o   = $s->wrap;
	my $dbh = $s->connect;

	my ( $params, $whereclause ) = $s->whereclause;

	my $sel = $dbh->prepare(
		qq{
		SELECT * FROM precursor $whereclause
	}
	) or die $dbh->errstr;

	$sel->execute(@$params) or die $sel->errstr;

	my $sth;
	if ( $sel->rows ) {
		unless ( $s->no_update ) {
			$sth = $dbh->prepare(
				qq{
				UPDATE precursor SET \
				cid = ?, 	\
				sequence = ?, 	\
				structure = ?,	\
				mfe = ?,	\
				start = ?,	\
				stop = ?,	\
				strand = ?,	\
				category = ?,	\
				score = ?,	\
				marked = ?,	\
				designation = ? \
				$whereclause
			}
			) or die $dbh->errstr;

			$sth->execute(
				$o->chromosome->id, $o->sequence,    $o->structure,
				$o->mfe,            $o->start,       $o->stop,
				$o->strand,         $o->category,    $o->score,
				$o->marked,         $o->designation, @$params
			) or die $sth->errstr;
		}
	}
	else {

		$sth = $dbh->prepare(
			qq{
			INSERT INTO precursor (cid, sequence, structure, mfe, start, stop, strand, category, score, marked, designation) \
			VALUES (?,?,?,?,?,?,?,?,?,?,?)
		}
		) or die $sth->errstr;

		$sth->execute(
			$o->chromosome->id, $o->sequence, $o->structure,
			$o->mfe,            $o->start,    $o->stop,
			$o->strand,         $o->category, $o->score,
			$o->marked,         $o->designation
		) or die $sth->errstr;

		my $id = $dbh->{q(mysql_insertid)};
		$o->set_id($id);
		$BAOBAB::Precursor::cache{$id} = $o unless $o->no_cache;
	}

	$dbh->commit or die $dbh->errstr;
	$sel->finish;
	$sth->finish unless $s->no_update;
	$s->clean;
}

sub list_candidates {
	my $s = shift;
	return $s->list( @_, category => 'candidate' );
}

sub list_known {
	my $s = shift;
	return $s->list( @_, category => 'known' );
}

sub list {
	my $s   = shift;
	my %a   = @_;
	my $dbh = $s->connect;

	my $organism   = $a{organism};
	my $chromosome = $a{chromosome};
	my $category   = $a{category};
	my $strand     = $a{strand};
	my $extra      = $a{extra};

	my $atom = qq{count(*)};
	$atom = qq{precursor.id} if wantarray;

	my $sql = qq{SELECT $atom FROM precursor};
	$sql .=
	  qq{, chromosome WHERE precursor.cid = chromosome.id AND chromosome.oid = }
	  . $organism->id
	  if defined $organism and !defined $chromosome;

	$sql .= qq{ WHERE precursor.cid =} . $chromosome->id if defined $chromosome;

	if ( defined $category ) {
		if ( !defined $organism and !defined $chromosome ) {
			$sql .= qq{ WHERE category = '$category'};
		}
		else {
			$sql .= qq{ AND category = '$category'};
		}
	}

	if ( defined $strand ) {
		if (    !defined $organism
			and !defined $chromosome
			and !defined $category )
		{
			$sql .= qq{ WHERE strand = '$strand'};
		}
		else {
			$sql .= qq{ AND strand = '$strand'};
		}
	}

	$sql .= qq{ $extra} if defined $extra;

	#print STDERR "SQL: $sql\n";
	my $res = $dbh->selectcol_arrayref($sql) or die $dbh->errstr;

	return wantarray ? @$res : $res->[0];
}

sub iterator {
	my $s     = shift;
	my $dbh   = $s->connect;
	my %a     = @_;
	my $start = $a{start};
	my $limit = $a{limit};
	my $extra = $a{extra};

	my $extraclause = qq{WHERE $extra} if defined $extra;

	if ( defined $start && defined $limit ) {
		$extraclause .= qq{ LIMIT $limit OFFSET $start};
	}

	$extraclause = '' unless defined $extraclause;
	my $select_clause = qq { SELECT id FROM precursor $extraclause };

	print STDERR $select_clause . "\n";

	my $sth = $dbh->prepare($select_clause) or die $dbh->errstr;
	$sth->execute();

	return sub {
		my ($id) = $sth->fetchrow_array();
		$sth->finish unless defined $id;
		return $id;
	};
}

sub read {
	my $s = shift;
	my %a = @_;

	my $id       = $a{id};
	my $no_cache = $a{no_cache};

	my $o = undef;
	if ( defined $id && exists $BAOBAB::Precursor::cache{$id} ) {
		$o = $BAOBAB::Precursor::cache{$id};
	}
	else {

		my $dbh = $s->connect;
		my ( $param, $whereclause ) = $s->whereclause(%a);

		my $data = $dbh->selectrow_hashref(
			qq{
				SELECT * FROM precursor $whereclause
			}, undef, @$param
		) or die $dbh->errstr;

		if ( exists $BAOBAB::Precursor::cache{ $data->{id} } ) {
			$o = $BAOBAB::Precursor::cache{ $data->{id} };
		}
		elsif ($data) {

			my $cw = BAOBAB::ChromosomeWrapper->new( dbh => $dbh );

			$o = BAOBAB::Precursor->new(
				id         => $data->{id},
				chromosome => sub {
					$cw->read(
						id       => $data->{cid},
						no_cache => $no_cache
					);
				},
				sequence    => $data->{sequence},
				structure   => $data->{structure},
				mfe         => $data->{mfe},
				start       => $data->{start},
				stop        => $data->{stop},
				strand      => $data->{strand},
				category    => $data->{category},
				score       => $data->{score},
				marked      => $data->{marked},
				designation => $data->{designation},
				no_cache    => $no_cache
			);
			$o->clean;
		}
	}

	return $s->wrap($o);
}

sub whereclause {
	my $s = shift;
	my $o = $s->wrap;
	my %a = @_;
	my ( $params, $whereclause ) = ( [], '' );

	my $no_params = scalar(@_) ? 0 : 1;

	if ( defined $a{id} or ( $no_params and defined $o and defined $o->id ) ) {
		$whereclause = q{WHERE id = ?};
		$params = [ $a{id} || $o->id ];
	}
	elsif (
		(
			    defined $a{chromosome}
			and defined $a{category}
			and defined $a{start}
			and defined $a{stop}
			and defined $a{strand}
		)
		or (
			    $no_params and defined $o and defined $o->chromosome,
			and defined $o->chromosome->id
			and defined $o->category
			and defined $o->start
			and defined $o->stop
			and defined $o->strand
		)
	  )
	{
		$whereclause =
q{WHERE cid = ? AND category = ? AND start = ? AND stop = ? AND strand = ?};

		my $cid = undef;
		$cid = $a{chromosome}->id if defined $a{chromosome};

		my $start = $a{start};
		$start = $o->start unless defined $start;
		my $stop = $a{stop};
		$stop = $o->stop unless defined $stop;

		$params = [
			$cid || $o->chromosome->id,
			$a{category} || $o->category,
			$start,
			$stop,
			$a{strand} || $o->strand
		];
	}
	else {
		$whereclause = q{WHERE 1 = 0};
	}

	return ( $params, $whereclause );

}

sub DESTROY {
	(shift)->SUPER::DESTROY(@_);
}

1;
