package MirBase::ParseName;

@ISA       = qw(Exporter);
@EXPORT_OK = qw(parse_name);
use Exporter;

sub parse_name {
	my $name   = shift;
	my $o      = {};
	my $plants = qr/^([a-z]+)-[mM][iI][rR](\d+[a-z]*)(\.\d)?(-[35]p)?\*?$/;
	my $animals =
	  qr/^([a-z]+)-([a-zA-Z]+)(-[0-9a-z]+)?(-\d+)?(-as)?(-[35]p)?\*?$/;
	my $virus =
qr/^([a-z][0-9a-z]+)-[mM][iI][rR]-([a-zA-Z0-9]+)-?(\d+[a-z]*)?(-[35]p)?(\.\d)?\*?$/;

	$o->{is_compliant} = 1;

	if ( $name =~ /$plants/ ) {
		$o->{sid}     = $1;
		$o->{number}  = $2;
		$o->{variant} = substr $3, 1 if $3;
		$o->{strand}  = substr $4, 1 if $4;
		$o->{name}    = 'MIR' . $2;
		$o->{name} .= $3 if $3;
		$o->{name} .= $4 if $4;
		$o->{mature_id} = "$1-MIR$2";
		$o->{kingdom}   = 'plantae';

	}
	elsif ( $name =~ /$animals/ ) {
		$o->{sid}    = $1;
		$o->{number} = substr $3, 1 if $3;
		$o->{locus}  = substr $4, 1 if $4;
		$o->{ori}    = $5 ? 'sense' : 'antisense';
		$o->{strand} = substr $6, 1 if $6;
		$o->{name}   = $2;
		$o->{name} .= $3 if $3;
		$o->{name} .= $4 if $4;
		$o->{name} .= $5 if $5;
		$o->{name} .= $6 if $6;
		$o->{mature_id} = "$1-$2";
		$o->{mature_id} .= $3 if $3;
		$o->{kingdom} = 'metazoa';

	}
	elsif ( $name =~ /$virus/ ) {
		$o->{sid}     = $1;
		$o->{number}  = $2;
		$o->{locus}   = $3;
		$o->{strand}  = substr $4, 1 if $4;
		$o->{variant} = substr $5, 1 if $5;
		$o->{name}    = 'mir-' . $2;
		$o->{name} .= '-' . $3 if $3;
		$o->{name} .= $4       if $4;
		$o->{name} .= $5       if $5;
		$o->{mature_id} = "$1-mir-$2";
		$o->{kingdom}   = 'viridae';

	}
	else {
		$o->{is_compliant} = 0;
	}

	#	assert_eq($name,$1.'-'.$o->{name});

	return wantarray ? %$o : $o;
}

sub assert_eq {
	my $one = shift;
	my $two = shift;

	die "$one ne $two" if uc($one) ne uc($two);

}

1;
