package BAOBAB::Fasta::Sequence;

# $Id: Sequence.pm,v 1.5 2009/10/12 15:29:55 rnl14480 Exp $

use BAOBAB::SequenceUtils;
use BAOBAB::Fasta;
use strict;

sub new {
	my $c = shift;
	my %a = @_;

	my $r = {
		_header => $a{header} || $a{seqid},
		_sequence => $a{sequence},
		_id       => $a{id},
		_dirty    => 1,
	};

	bless $r, $c;

	if ( $a{save_memory} ) {
		$r->{_length} = length( $r->{_sequence} ) if defined $r->{_sequence};
		$r->{_sequence} = undef;
	}
	return $r;

}

sub header   { return (shift)->{_header}; }
sub id       { return (shift)->{_id}; }
sub sequence { return (shift)->{_sequence}; }
sub is_dirty { return (shift)->{_dirty}; }
sub clean    { return (shift)->{_dirty} = 0; }
sub dirty    { return (shift)->{_dirty} = 1; }

sub length {
	my $s = shift;
	unless ( defined $s->{_length} ) {
		$s->{_length} = length( $s->{_sequence} );
	}

	return $s->{_length};
}

sub _init_seqstats { }

sub get_region {
	my $s = shift;
	my %a = @_;

	return undef unless defined $s->{_sequence};

	my $start  = $a{start};
	my $stop   = $a{stop};
	my $strand = $a{strand} || 1;

	my $out = substr $s->{_sequence}, $start - 1, $stop - $start + 1;
	$out = reverse_complement($out) if $strand < 0;

	return $out;

}

1;
